@@ -7,7 +7,8 @@ def add_contact(name, address, step, **kwargs):
     redis_client = kwargs.get('redis_client')
     executor =  kwargs.get('executor')
     providers = kwargs.get("providers")
-    provider = Web3.HTTPProvider(providers['ethereum'])
+    rpc_url = providers['ethereum']['rpc']
+    provider = Web3.HTTPProvider(rpc_url)
     provider.middlewares.clear()
     w3 = Web3(provider)
     ens_name = None
