@@ -221,71 +221,62 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
     """
       kwargs need to include providers from different chains
       for example: 
-      providers: {
-        "base": "https://base.g.alchemy.com/v2/7mj7kTlPr2M1AiMu42Z3Tbwwg8DyOHPk", 
-        "ethereum": "https://ethereum.g.alchemy.com/v2/7mj7kTlPr2M1AiMu42Z3Tbwwg8DyOHPk" 
-      }
+      "providers": { 
+        "base": {
+            "rpc": configs.base_rpc_url,
+            "scan": configs.base_scan_url,
+        }, 
+        "ethereum": {
+            "rpc": configs.eth_rpc_url,
+            "scan": configs.eth_scan_url
+        },
+        "eth": {
+            "rpc": configs.eth_rpc_url,
+            "scan": configs.eth_scan_url
+        },
+        "bsc": {
+            "rpc": configs.bsc_rpc_url,
+            "scan": configs.bsc_scan_url
+        },
+      },
     """
     print(f"receiver: {receiver}, amount: {amount}, run_mode: {run_mode}, chain: {chain}, token: {token}, kwargs: {kwargs}, step: {step}")
     info = None
+    chain_raw = chain.lower()
     executor = kwargs.get("executor")
     redis_client = kwargs.get("redis_client")
     control_address = kwargs.get('control_address')
     control_contract = kwargs.get('contract')
     private_key = kwargs.get('private_key')
     spender_address =  kwargs.get('spender_address')
-    blockchain_explore = kwargs.get('blockchain_explore')
     providers = kwargs.get("providers")
-    provider = Web3.HTTPProvider(providers[chain.lower()])
+    
+    if not providers or chain_raw not in providers:
+        raise ValueError(f"Provider for chain {chain} not found")
+    
+    rpc_url = providers[chain_raw]['rpc']
+    blockchain_explore = providers[chain_raw]['scan']
+
+    provider = Web3.HTTPProvider(rpc_url)
     provider.middlewares.clear()
     w3 = Web3(provider)
-
     token_raw = token
-    token = w3.to_checksum_address(token_mapping[chain.lower()][token.lower()])
-
-    print(f"token: {token}, token_raw: {token_raw}")
+    token = w3.to_checksum_address(token_mapping[chain_raw][token_raw.lower()])
 
     if run_mode == 'auto':
       """ check executor approved allowance """
-      # token = w3.to_checksum_address(token_mapping[chain.lower()][token.lower()]) 
       token_contract = w3.eth.contract(address=token, abi=allowance_abi)
       allowance = token_contract.functions.allowance(Web3.to_checksum_address(executor), spender_address).call()
       allowance_normalized = Web3.from_wei(allowance, 'mwei')
       print(f"allowance_normalized: {float(allowance_normalized)}")
       if float(allowance_normalized) < float(amount):
-          # active_chain_key = f"user:{executor}:active_chain"
-          # active_chain = redis_client.get(active_chain_key)
-          # if not active_chain:
-          #   active_chain = "base"
-          # if active_chain.lower() != chain.lower():
-          #   action_params = {
-          #     "func": "wallet_switchEthereumChain",
-          #     "chainId": chain_id_mapping[chain.lower()]
-          #   }
-          #   action_data = ActionData(func='', group='', params=action_params)
-          #   action = Action(msg=None, type='setting', data=action_data.__dict__)
-          #   next_params = {
-          #     'receiver': receiver,
-          #     'amount': amount,
-          #     'chain': chain,
-          #     'token': token_raw,
-          #     'run_mode': run_mode
-          #   }
-          #   next_action_data = ActionData(func='transfer', group='transfer', params=next_params)
-          #   next_action = Action(msg=None, type='backend', data=next_action_data.__dict__)
-
-          #   return {
-          #     'result': { 'code': 403, 'content': 'Incorrect network, switching to the correct network for you.' },
-          #     'action': action.__dict__,
-          #     'next_action': next_action.__dict__
-          #   }
-          # else:
+          amount_to_transfer = w3.to_wei(float(amount), 'mwei')
           action_params = {
             "func": "approve",
-            "chainId": chain_id_mapping[chain.lower()],
+            "chainId": chain_id_mapping[chain_raw],
             'contract': token,
             '_spender': spender_address,
-            '_value': float(amount),
+            '_value': amount_to_transfer,
             'abi': erc20_approve_abi
           }
           action_data = ActionData(func='', group='', params=action_params)
@@ -305,7 +296,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
           next_params = {
             'receiver': to,
             'amount': amount,
-            'chain': chain,
+            'chain': chain_raw,
             'token': token_raw,
             'run_mode': run_mode
           }
@@ -354,12 +345,11 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
               status = 500
               message = "failed"
           info = {
-            "content":
-            generate_markdown_content(tx_receipt['status'], tx_hash, blockchain_explore),
-            "status":
-            tx_receipt['status'],
-            "code":
-            status
+            "content": generate_markdown_content(tx_receipt['status'], tx_hash, blockchain_explore),
+            "tx_hash": tx_hash,
+            "chain": chain_raw,
+            "status": tx_receipt['status'],
+            "code": status
           }
           return { 
             'result': info, 
@@ -387,10 +377,10 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
       active_chain = redis_client.get(active_chain_key)
       if not active_chain:
         active_chain = "base"
-      if active_chain.lower() != chain.lower():
+      if active_chain.lower() != chain_raw:
           params = {
             "func": "wallet_switchEthereumChain",
-            "chainId": chain_id_mapping[chain.lower()]
+            "chainId": chain_id_mapping[chain_raw]
           }
           action_data = ActionData(func='', group='', params=params)
           action = Action(msg=None, type='setting', data=action_data.__dict__)
@@ -409,7 +399,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
           params = {
             'receiver': to,
             'amount': amount,
-            'chain': chain,
+            'chain': chain_raw,
             'token': token_raw,
             'run_mode': run_mode
           }
@@ -436,8 +426,8 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
           to = w3.to_checksum_address(receiver)
           params = {
             'recipient': to,
-            'amount': amount,
-            'chainId': chain_id_mapping[chain.lower()],
+            'amount': amount_to_transfer,
+            'chainId': chain_id_mapping[chain_raw],
             'contract': token,
             'func': 'transfer',
             'abi': erc20_transfer_abi
@@ -445,7 +435,7 @@ def transfer(receiver, amount, step, run_mode = 'auto', chain = "base", token =
           action_data = ActionData(func='transfer', group='transfer', params=params)
           action = Action(msg=None, type='wallet', data=action_data.__dict__)
           return {
-            'result': { "code": 200, "content": "" },
+            'result': { "code": 200, "content": "Manual transfer parameters are constructed" },
             'action': action.__dict__,
             'next_action': None
           }
