@@ -35,7 +35,7 @@ def redis_show_contact(redis_client, user, name):
     res = None
     try:
         prefix = f"contact:{user}"
-        name_address_map = get_name_address_mapping(redis_client, prefix)
+        name_address_map = get_user_contacts_mapping(redis_client, prefix)
         if name is not None:
             name_address_map = { name: name_address_map[name] }
         res = generate_markdown(name_address_map)
@@ -44,18 +44,25 @@ def redis_show_contact(redis_client, user, name):
         return 500, res
     return 200, res
 
-def get_name_address_mapping(redis_client, prefix: str):
-    name_address_map = {}
+def get_user_contacts_mapping(redis_client, prefix: str):
+    contact_mapping = {}
     for key in redis_client.scan_iter(match=f"{prefix}*"):
         # key: contact:user_address_name
         name = key.split('_')[-1]
         address = redis_client.hget(key, 'address')
+        ens_name = redis_client.hget(key, 'ens_name')
         if address:
-            name_address_map[name] = address
-    return name_address_map
+            # name_address_map[name] = address
+            contact_mapping[name] = {
+                "address": address,
+                "ens_name": ens_name if ens_name is not None else "NULL"
+            }
+    return contact_mapping
 
 def generate_markdown(mapping):
-    markdown_lines = ["| Name | Address |", "|------|---------|"]
-    for name, address in mapping.items():
-        markdown_lines.append(f"| {name} | {address} |")
+    markdown_lines = ["| Name | Address | ENS |", "|------|---------|----------|"]
+    for name, details in mapping.items():
+        address = default_if_none(details['address'], 'NULL')
+        ens_name = default_if_none(details['ens_name'], 'NULL')
+        markdown_lines.append(f"| {name} | {address} | {ens_name} |")
     return "\n".join(markdown_lines)
\ No newline at end of file
